/*==============================================================================
   DEMO.C :- FORM Demo Program (MS Windows Version)
   Sub Systems, Inc.
   Software License Agreement  (1989)              
   ----------------------------------
   This is a demo program.  The program shows how you can utilize the Report 
   Ease routines to perform develop report templates and execute the reports.
===============================================================================*/

#include "windows.h"
#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "sys\types.h"
#include "sys\stat.h"
#include "io.h"
#include "dos.h"
#include "string.h"
#include "math.h"

#if defined(__TURBOC__)
   #include "dir.h"
   #include "mem.h"
#else
   #include "memory.h"
#endif

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if defined (WIN32)
   #undef  huge
   #define huge
   #undef  FreeProcInstance 
   #define FreeProcInstance(x) 
   #define COMMAND_ID(wp,lp)        LOWORD(wp)
   #define CONTROL_ID(wp,lp)        LOWORD(wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(wp)
   #undef  farmalloc
   #define farmalloc  malloc
   #undef  farrealloc
   #define farrealloc realloc
   #undef  farfree
   #define farfree    free
#else
   #define COMMAND_ID(wp,lp)        (wp)
   #define CONTROL_ID(wp,lp)        (wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(lp)
#endif

/******************************************************************************
    DLL and demo program specific includes
*******************************************************************************/
#include "frs.h"
#include "demo.h"

/****************************************************************************
        Main Fuction
******************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;

    hPrevInst=hPrevInstance;
    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    
    return (msg.wParam);
}

/****************************************************************************
   Initialize window data and register window class
****************************************************************************/

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS  wc;

    wc.style = 0;
    wc.lpfnWndProc = (void far *)DemoWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName =  "DemoMenu";
    wc.lpszClassName = "FormDemoClass";

    return (RegisterClass(&wc));
}


/****************************************************************************
    Save instance handle and create main window
****************************************************************************/

BOOL InitInstance(HANDLE hInstance,int nCmdShow)
{
    HWND            hDemoWin;
    
    hInst = hInstance;

    hDemoWin = CreateWindow(
        "FormDemoClass",
        "Form Plus Demo",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL
    );

    if (!hDemoWin)
        return (FALSE);

    ShowWindow(hDemoWin, nCmdShow);
    UpdateWindow(hDemoWin);
    
    //****** initialize the structure variables *********
    FormParm.open=FALSE;           // all windows closed in the beginning 
    FormParm.InDesignMode=TRUE;      // design mode by default
    FormParm.x=0;                  // Initial X position of FORM edit window 
    FormParm.y=0;                  // Initial Y position of FORM edit window 
    FormParm.width=GetSystemMetrics(SM_CXSCREEN);        // Initial edit window width 
    FormParm.height=(GetSystemMetrics(SM_CYSCREEN)*9)/10; // Initial edit window height 
    FormParm.ShowMenu=TRUE;        // display menu 
    FormParm.ShowVerBar=TRUE;      // display vertical scroll bar 
    FormParm.ShowHorBar=TRUE;      // display horizontal scroll bar (N/A with word wrap) 
    FormParm.ShowToolbar=TRUE;     // show toolbar icons
    strcpy(FormParm.file,"");      // Text file name if the input type is 'F' 
    FormParm.hInst=hInst;          // Current application instant handle 
    FormParm.hPrevInst=hPrevInst;  // Previous application instant handle 
    FormParm.hParentWnd=hDemoWin;  // let this be the parent of the editor window 
    FormParm.style=WS_OVERLAPPEDWINDOW; //  Editor window style 
    strcpy(FormParm.DataSetName,"CUSTOMER");


    //***** pass the pointer to the call back routines 
    /** The field selection and verification routines must be included in the 
        export section of your applications definition file **/

    FormParm.UserSelection=(USER_SELECTION)MakeProcInstance(UserFieldSelection,hInst); // field selection routine 
    if (!FormParm.UserSelection) return FALSE;
    FormParm.VerifyData=(VERIFY_DATA)MakeProcInstance(VerifyData,hInst); // data verification callback function
    if (!FormParm.VerifyData) return FALSE;
    FormParm.DrawPicture=(DRAW_PICTURE)MakeProcInstance(DrawPicture,hInst); // data verification callback function
    if (!FormParm.DrawPicture) return FALSE;
    FormParm.EnterData=(ENTER_DATA)MakeProcInstance(EnterData,hInst); // data verification callback function
    if (!FormParm.EnterData) return FALSE;

    //********** read field name for the files *******************
    if (!ReadFields()) return FALSE;   // read the field defintion

    // load the customer logo bitmap
    hLogoBM=LoadBitmap(hInst,"LogoBM");
    Bitmap2DIB();                      // extract bits from the bitmap

    return (TRUE);
}

/****************************************************************************
    Process Main Window messages
****************************************************************************/

long FAR PASCAL _export DemoWndProc(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
    LPVOID lpProcParam;


    switch (message) {
        case WM_COMMAND:
           switch (COMMAND_ID(wParam,lParam)) {
              case IDM_EDITOR:                // open a new window
                CallEditor(TRUE);             // call the form editor 
                break;
            
              case IDM_EXECUTER:              // run a form template 
                CallEditor(FALSE);            // run the report 
                break;
            
              case IDM_OPTIONS:          // accept editor window options 
                lpProcParam = MakeProcInstance(DemoOptions, hInst);
                DialogBox(hInst,"DemoOptions",hWnd,lpProcParam);
                FreeProcInstance(lpProcParam);
                break;
            
              case IDM_EXIT:             // quit demo 
                if (FormParm.open) {
                   if (!SendMessage(FormParm.hFrWnd,WM_QUERYENDSESSION,0,0L)) break;
                }
                PostQuitMessage(0);

              default:
                return (DefWindowProc(hWnd, message, wParam, lParam));
           }
           break;

        case WM_DESTROY:
            if (FormParm.open) {
               if (!SendMessage(FormParm.hFrWnd,WM_QUERYENDSESSION,0,0L)) break;
            }
            
            // release demo bitmap data
            if (hLogoBM) DeleteObject(hLogoBM);
            GlobalUnlock(hImage);
            GlobalUnlock(hInfo);
            GlobalFree(hImage);
            GlobalFree(hInfo);

            PostQuitMessage(0);
            break;

        /********************************************************************
                      Messages coming from the form editor Window
        *********************************************************************/
        case FRS_CLOSE:                      // REP sends this message before closing 
            FormParm.open=FALSE;             // mark Report Ease as closed 
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (LRESULT)(NULL);
}

/****************************************************************************
    CallEditor:
    This function calls the form editor.
****************************************************************************/
CallEditor(BOOL InDesignMode)
{
    HWND hWnd;

    if (GetFormSelection(FormParm.file,InDesignMode)>=0) {  // select a form file 
       
       //*** let form designer use the default fonts ******
       FormParm.FontTypeFace[0]=0;     // default font type faces for report 
       FormParm.InDesignMode=InDesignMode; // design mode

       hWnd=FrsForm(&FormParm);       // call the form editor 
       
       if (!hWnd) {                   // print error if any 
          MessageBox(NULL,"Error calling the form editor",NULL,MB_OK);
       }
    }

    return TRUE;
}

/****************************************************************************
    DemoOptions:
    This dialog box allows the user to change the editor window options.
****************************************************************************/
BOOL FAR PASCAL _export DemoOptions(HWND hDlg,unsigned message,WPARAM wParam,LPARAM lParam)
{
    switch (message) {
        case WM_INITDIALOG:
            // Set initial values of the parameters 
            SetDlgItemInt(hDlg,IDC_X,FormParm.x,0);
            SetDlgItemInt(hDlg,IDC_Y,FormParm.y,0);
            SetDlgItemInt(hDlg,IDC_WIDTH,FormParm.width,0);
            SetDlgItemInt(hDlg,IDC_HEIGHT,FormParm.height,0);
            SendMessage(GetDlgItem(hDlg,IDC_MENU),BM_SETCHECK, FormParm.ShowMenu, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_TOOLBAR),BM_SETCHECK, FormParm.ShowToolbar, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_VER_BAR),BM_SETCHECK, FormParm.ShowVerBar, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_HOR_BAR),BM_SETCHECK, FormParm.ShowHorBar, 0L);
            return (TRUE);

        case WM_COMMAND:
            switch (CONTROL_ID(wParam,lParam)) {
                case IDOK:
                   if (!GetNumVal(hDlg,IDC_X,&FormParm.x)) return (TRUE);
                   if (!GetNumVal(hDlg,IDC_Y,&FormParm.y)) return (TRUE);
                   if (!GetNumVal(hDlg,IDC_WIDTH,&FormParm.width)) return (TRUE);
                   if (!GetNumVal(hDlg,IDC_HEIGHT,&FormParm.height)) return (TRUE);
                   FormParm.ShowMenu=SendMessage(GetDlgItem(hDlg,IDC_MENU),BM_GETCHECK, 0, 0L);
                   FormParm.ShowToolbar=SendMessage(GetDlgItem(hDlg,IDC_TOOLBAR),BM_GETCHECK, 0, 0L);
                   FormParm.ShowVerBar=SendMessage(GetDlgItem(hDlg,IDC_VER_BAR),BM_GETCHECK, 0, 0L);
                   FormParm.ShowHorBar=SendMessage(GetDlgItem(hDlg,IDC_HOR_BAR),BM_GETCHECK, 0, 0L);
                   EndDialog(hDlg, TRUE);
                   return (TRUE);
        
                case IDCANCEL:
                   EndDialog(hDlg, FALSE);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/******************************************************************************
    GetFormSelection:
    Shows available report forms and let user select one.
    In form edit mode, the user has two additional selections to
    create a new report form.
******************************************************************************/
GetFormSelection(char *CurFile,int FormEdit)
{
     int  select;
     LPVOID lpProcParam;

     TotalForms=0;
     
     strcpy(CurFile," ");
     
     #if defined (WIN32)
        GetFormFiles("*.FIC");          // get the report form files 
     #else
        GetFormFiles("*.FI");          // get the report form files 
     #endif

     if (FormEdit) {                // allow the user to create new forms 
         NewReport=TotalForms;      // append the new report selection 
         TotalForms++;
         strcpy(FormName[NewReport],"New Form");
         strcpy(FormFile[NewReport]," ");
     }
     else NewReport=TotalForms;

     //***** show the file seletion box and let user select a file *****
     lpProcParam = MakeProcInstance(DemoForms, hInst);
     select=DialogBox(hInst,"DemoForms",hDemoWin,lpProcParam);
     FreeProcInstance(lpProcParam);

     if (select>=0) {
        strcpy(CurFile,"");
        if (select!=NewReport) strcpy(CurFile,FormFile[select]);  // pass the file name 
     }

     return select;
}

/****************************************************************************
    DemoForms:
    This dialog box allows the user to select a form file.
****************************************************************************/

BOOL FAR PASCAL _export DemoForms(HWND hDlg,unsigned message,WPARAM wParam,LPARAM lParam)
{
    int i,idx;

    switch (message) {
        case WM_INITDIALOG:
            // Set initial values of the parameters 
            for (i=0;i<TotalForms;i++) {
               idx=(int)SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_ADDSTRING, 0, (DWORD)(LPSTR)FormName[i]);
               SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_SETITEMDATA, idx, i);
            }
            SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_SETCURSEL, 0,0L);
            SetFocus(GetDlgItem(hDlg,IDC_FILE_BOX));
            return (FALSE);

        case WM_COMMAND:
            switch (CONTROL_ID(wParam,lParam)) {
                case IDC_FILE_BOX:
                    if (NOTIFY_MSG(wParam,lParam)!=LBN_DBLCLK) break;

                case IDOK:
                   i=SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_GETCURSEL, 0,0L);     // index into the list box
                   i=SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_GETITEMDATA, i,0L);   // index into our table
                   EndDialog(hDlg, i);
                   return (TRUE);
        
                case IDCANCEL:
                   EndDialog(hDlg, -1);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/****************************************************************************
    DemoFileField:
    This dialog box allows the user to select a data file or to select
    a field for a data file.  When lParam is -1, the dialog box shows 
    the data files to select from.  When lParam is zero or greater than 
    zero, the dialog box shows the fields for the file to select from.  The
    lParam, then specifies the index of the data file to choose the
    fields from.
****************************************************************************/

int FAR PASCAL _export DemoFileField(HWND hDlg,unsigned message,WPARAM wParam,LPARAM lParam)
{
    int i;

    switch (message) {
        case WM_INITDIALOG:
            // Set initial values of the parameters 
            for (i=0;i<TotalFields;i++) {
               SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_ADDSTRING, 0, (DWORD)(LPSTR)FieldDef[i].ShortName);  // load the field names 
            }
            SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_SETCURSEL, 0,0L);
            SetFocus(GetDlgItem(hDlg,IDC_FILE_BOX));
            return (FALSE);

        case WM_COMMAND:
            switch (CONTROL_ID(wParam,lParam)) {
                case IDC_FILE_BOX:
                    if (NOTIFY_MSG(wParam,lParam)!=LBN_DBLCLK) break;

                case IDOK:
                   i=SendMessage(GetDlgItem(hDlg,IDC_FILE_BOX),LB_GETCURSEL, 0,0L);
                   EndDialog(hDlg, i);
                   return (TRUE);
        
                case IDCANCEL:
                   EndDialog(hDlg, -1);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/****************************************************************************
    DemoRating:
    This dialog box allows the user to select the customer rating.
****************************************************************************/
BOOL FAR PASCAL _export DemoRating(HWND hDlg,unsigned message,WPARAM wParam,LPARAM lParam)
{
    int i,sel;

    switch (message) {
        case WM_INITDIALOG:
            // Set the list box 
            SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_ADDSTRING, 0, (DWORD)(LPSTR)"Rated High");
            SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_ADDSTRING, 0, (DWORD)(LPSTR)"Rated Medium");
            SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_ADDSTRING, 0, (DWORD)(LPSTR)"Rated Low");
            SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_ADDSTRING, 0, (DWORD)(LPSTR)"Not Rated");
            
            // set the current selection
            if      (lstrcmpi(DlgText,"A")==0) sel=0;
            else if (lstrcmpi(DlgText,"B")==0) sel=1;
            else if (lstrcmpi(DlgText,"C")==0) sel=2;
            else                               sel=3;

            SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_SETCURSEL, sel,0L);
            SetFocus(GetDlgItem(hDlg,IDC_BOX));
            
            // move the window to the given x/y location
            SetWindowPos(hDlg,NULL,DlgInt1,DlgInt2,0,0,SWP_NOSIZE|SWP_NOZORDER);
            
            return (FALSE);

        case WM_COMMAND:
            switch (CONTROL_ID(wParam,lParam)) {
                case IDC_BOX:
                    if (NOTIFY_MSG(wParam,lParam)!=LBN_DBLCLK) break;

                case IDOK:
                   i=SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_GETCURSEL, 0,0L);
                   
                   if      (i==0) lstrcpy(DlgText,"A");
                   else if (i==1) lstrcpy(DlgText,"B");
                   else if (i==2) lstrcpy(DlgText,"C");
                   else           lstrcpy(DlgText,"X");

                   EndDialog(hDlg,TRUE);
                   return (TRUE);
        
                case IDCANCEL:
                   EndDialog(hDlg, FALSE);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/******************************************************************************
    GetFormFiles:
    This routine scans the current directory for the files matching the
    specified template.
******************************************************************************/
GetFormFiles(char *template)
{
    unsigned done;
    FILE *iStream;
    struct StrFormHdr hdr;

    #if defined (WIN32) 
       HANDLE hFindFile;
       WIN32_FIND_DATA blk;
       done=FALSE;
       if (INVALID_HANDLE_VALUE==(hFindFile=FindFirstFile(template,&blk))) done=TRUE;
    #else
       #if defined(__TURBOC__)
          struct ffblk blk;
          done=findfirst(template,&blk,FA_RDONLY);
       #else
          struct find_t blk;
          done=_dos_findfirst(template,_A_NORMAL,&blk);
       #endif
    #endif

    
    while(!done) {
       if (TotalForms==MAX_FORMS) {
          return FALSE;
       }

       #if defined (WIN32)
          strcpy(FormFile[TotalForms],blk.cFileName);
          done=!FindNextFile(hFindFile,&blk);
       #else
          #if defined(__TURBOC__)
             strcpy(FormFile[TotalForms],blk.ff_name);
             done=findnext(&blk);         // get the next file 
          #else
             strcpy(FormFile[TotalForms],blk.name);
             done=_dos_findnext(&blk);
          #endif
       #endif
       
       //*************** get the form name ****************************
       if (NULL==(iStream=fopen(FormFile[TotalForms],"r+b"))) continue;
       
       if (fread(&hdr,sizeof(struct StrFormHdr),1,iStream)!=1
          || hdr.FormSign!=FORM_SIGN ) {
           fclose(iStream);
           continue;
       }
       fclose(iStream);

       strcpy(FormName[TotalForms],hdr.name);

       TotalForms++;
    }

    return TRUE;
}

/******************************************************************************
    Database access functions.
    The demo program uses a flat ascii file to store the customer records.
    The following functions provide access to this database.  Your application
    will typically use the access functions provided by the database that you
    are using.
******************************************************************************/

/******************************************************************************
    ShowRecord:
    Show the field values for the current record.  The current record has
    been read into the FieldDef array.
******************************************************************************/
BOOL ShowRecord(HWND hWnd, BOOL initialize)
{
    int i;

    for (i=0;i<TotalFields;i++) {
       if (initialize) {            // initialize the field value
          FieldDef[i].NumData=0;
          FieldDef[i].DblData=0;
          FieldDef[i].TextData[0]=0;
       }
       FrsSetFieldData(hWnd,NULL,i,FieldDef[i].NumData,FieldDef[i].DblData,FieldDef[i].TextData,FALSE);
    }
    FrsRepaint(hWnd,PAINT_WIN);    // repaint the window

    return TRUE;
}

/******************************************************************************
    WriteRecord:
    This function adds, update or delete a customer record.
    This function reads the records from the current database and copies them
    to the new database.  In the process it also adds a new record or updates
    an existing record.
******************************************************************************/
BOOL WriteRecord(HWND hWnd, LPSTR CustId, char action)
{
    char NewFile[64],BkupFile[64];
    FILE *iStream;
    FILE *oStream;
    int i,KeyField;
    BOOL add=FALSE,update=FALSE,delete=FALSE;
    BOOL added=FALSE,updated=FALSE,deleted=FALSE,RecordRead;
    struct StrFieldDef SaveField[MAX_FIELDS];

    // check the input operation
    if      (action=='A') add=TRUE;
    else if (action=='U') update=TRUE;
    else if (action=='D') delete=TRUE;

    // open the current database  
    if (NULL==(iStream=fopen(DataFile,"r+t"))) return FALSE;  // open the input file
    
    strcpy(NewFile,DataFile);
    ModifyExt(NewFile,".NEW");
    unlink(NewFile);
    if (NULL==(oStream=fopen(NewFile,"wt"))) return FALSE; // open the output file
    
    KeyField=0;     // the index of the customer id field is 0

    while ((RecordRead=ExtractNextRecord(iStream))==TRUE || (add && !added)) {
       // add the new record
       if ((lstrcmpi(FieldDef[KeyField].TextData,CustId)>=0 || !RecordRead) && add && !added) {
           // save the current record
           if (RecordRead) for (i=0;i<TotalFields;i++) SaveField[i]=FieldDef[i];

           // Get the current values from the screen
           for (i=0;i<TotalFields;i++) {
              if (!FrsGetFieldData(hWnd,NULL,i,&(FieldDef[i].NumData),&(FieldDef[i].DblData),FieldDef[i].TextData)) {
                 FieldDef[i].NumData=0;       // initialize the current 
                 FieldDef[i].DblData=0;
                 FieldDef[i].TextData[0]=0;
              }
           }
           if (!WriteCurRecord(oStream)) return FALSE; // write the new record
           ShowRecord(hWnd,TRUE);    // clear the field values from the screen
           added=TRUE;
           if (!RecordRead) break;   // end of the input file;

           // now restore the input record
           if (RecordRead) for (i=0;i<TotalFields;i++) FieldDef[i]=SaveField[i];
       }

       // update the current record
       else if (lstrcmpi(FieldDef[KeyField].TextData,CustId)==0 && update && !updated) {
           // Get the field values from the screen
           for (i=0;i<TotalFields;i++) {
              FrsGetFieldData(hWnd,NULL,i,&(FieldDef[i].NumData),&(FieldDef[i].DblData),FieldDef[i].TextData);
           }
           updated=TRUE;
       }

       // update the current record
       else if (lstrcmpi(FieldDef[KeyField].TextData,CustId)==0 && delete) {
           // show a blank record
           ShowRecord(hWnd,TRUE);
           deleted=TRUE;
           continue;
       }

       // write the current record
       if (!WriteCurRecord(oStream)) return FALSE; // write current record to the output file
    }

    fclose(iStream);
    fclose(oStream);

    // rename the files
    lstrcpy(BkupFile,DataFile);
    ModifyExt(BkupFile,".BU");
    unlink(BkupFile);
    if (rename(DataFile,BkupFile)) return FALSE;
    if (rename(NewFile,DataFile)) return FALSE;

    if (added || updated || deleted) return TRUE; 
    else                             return FALSE;
}

/******************************************************************************
    WriteCurRecord:
    Write the current record to the output file.
******************************************************************************/
BOOL WriteCurRecord(FILE *oStream)
{
    char line[300]="",quote=34,QuoteString[2],string[20];
    int i,len;

    // buile a string containg a quote character
    QuoteString[0]=quote;
    QuoteString[1]=0;

    // build the text line to write 
    for (i=0;i<TotalFields;i++) {
       // write the comma delimiter
       if (i>0) {
          int len=lstrlen(line);
          line[len]=',';
          line[len+1]=0;
       }
       
       // append a field data string
       if (FieldDef[i].type==TYPE_TEXT) {
          lstrcat(line,QuoteString);
          lstrcat(line,FieldDef[i].TextData);
          lstrcat(line,QuoteString);
       }
       else if (FieldDef[i].type==TYPE_RADIO || FieldDef[i].type==TYPE_CHECKBOX || FieldDef[i].type==TYPE_PUSHBUTTON) {
          if (FieldDef[i].NumData) lstrcat(line,"Y");
          else                     lstrcat(line,"N");
       }
       else if (FieldDef[i].type==TYPE_DBL) { // append the double data
          DoubleToStr(FieldDef[i].DblData,FieldDef[i].DecPlaces,string);
          lstrcat(line,string);
       }
       else {
          wsprintf(string,"%ld",FieldDef[i].NumData);
          lstrcat(line,string);
       }
    }

    // add the new line character and write to the file
    len=lstrlen(line);
    line[len]=0xa;
    line[len+1]=0;

    if (fputs(line,oStream)<0) return FALSE;

    return TRUE;
}

/******************************************************************************
    ReadRecord:
    Read a record from the specified customer id. If the record is found,
    its fields are read into the StrFieldDef structure.
******************************************************************************/
BOOL ReadRecord(LPSTR CustId)
{
    char name[64];
    FILE *iStream;
    int KeyField;
    BOOL RecFound=FALSE;

    strcpy(name,DataFile);
    if (NULL==(iStream=fopen(name,"r+t"))) return FALSE;

    KeyField=0;     // the index of the customer id field is 0

    while (ExtractNextRecord(iStream)) {
       if (lstrcmpi(FieldDef[KeyField].TextData,CustId)==0) {
          RecFound=TRUE;
          break;
       }
    }

    fclose(iStream);

    return RecFound;              // record not found
}

/******************************************************************************
    ReadNextRecord:
    Read the next from the file. If the record is found,
    its fields are read into the StrFieldDef structure.
******************************************************************************/
BOOL ReadNextRecord(LPSTR CustId)
{
    char name[64];
    FILE *iStream;
    int KeyField;
    BOOL RecFound=FALSE;

    strcpy(name,DataFile);
    if (NULL==(iStream=fopen(name,"r+t"))) return FALSE;

    KeyField=0;     // the index of the customer id field is 0

    while (ExtractNextRecord(iStream)) {
       if (lstrcmpi(FieldDef[KeyField].TextData,CustId)>0) {
          RecFound=TRUE;
          break;
       }
    }

    fclose(iStream);

    return RecFound;              // record not found
}

/******************************************************************************
    ReadPrevRecord:
    Read the previous record from the file. If the record is found,
    its fields are read into the StrFieldDef structure.
******************************************************************************/
BOOL ReadPrevRecord(LPSTR CustId)
{
    char name[64],PrevCustId[10]="";
    FILE *iStream;
    int KeyField;

    strcpy(name,DataFile);
    if (NULL==(iStream=fopen(name,"r+t"))) return FALSE;

    KeyField=0;     // the index of the customer id field is 0

    while (ExtractNextRecord(iStream)) {
       if (lstrcmpi(FieldDef[KeyField].TextData,CustId)>=0) break;
       lstrcpy(PrevCustId,FieldDef[KeyField].TextData);   // save the previous key
    }

    fclose(iStream);

    if (lstrlen(PrevCustId)==0) return FALSE;

    return ReadRecord(PrevCustId);   // return the previous customer id
}

/******************************************************************************
    ExtractNextRecord:
    Extract next record from the customer file.
******************************************************************************/
BOOL ExtractNextRecord(FILE *iStream)
{
    char line[300],string[300],*result;
    int  CurLen,i,LineIdx;

    result=fgets(line,300,iStream);
    if (ferror(iStream)) {
       wsprintf(line,"Error while reading the data file");
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    if (feof(iStream))   return FALSE; // end of file 
    if (result==NULL) {
       wsprintf(line,"Error while reading the data file");
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    
    CurLen=strlen(line);             // exclude new line character 
    if (CurLen>0 && line[CurLen-1]==0xA) CurLen--;
    line[CurLen]=0;

    LineIdx=0;                       // prepare to scan the line 

    // scan the line to extract the data for each field
    for (i=0;i<TotalFields;i++) {
      ExtractField(line,&LineIdx,CurLen,string);  // return the data in the text format

      // translate from text to appropriate field type
      if (FieldDef[i].type==TYPE_TEXT)     lstrcpy(FieldDef[i].TextData,string);
      else if (FieldDef[i].type==TYPE_DBL) FieldDef[i].DblData=(double)atof(string);
      else if (FieldDef[i].type==TYPE_NUM) FieldDef[i].NumData=atol(string); // date shoule be YYMMDD or YYYYMMDD 
      else if (FieldDef[i].type==TYPE_DATE)FieldDef[i].NumData=atol(string); // date shoule be YYMMDD or YYYYMMDD 
      else if (FieldDef[i].type==TYPE_PICT)FieldDef[i].NumData=atol(string); // date shoule be YYMMDD or YYYYMMDD 
      else {                                 // button fields
         if (string[0]=='Y' || string[0]=='y') FieldDef[i].NumData=1;
         else                                  FieldDef[i].NumData=0;
      }
    }

    return TRUE;
}


/******************************************************************************
    ReadFields:
    Read the field definitions for a file.
    The field definitions are stored in files with the extension .DF, whereas
    the data is stored in the files with the extension .DB.  
******************************************************************************/
ReadFields()
{
    char name[64],line[300],CharReturn[300],*result;
    FILE *iStream;
    int  CurLen,CurField=0,LineIdx;
    int  ExtractField(char *,int *,int,char *);

    strcpy(name,DataDefFile);

    iStream=fopen(name,"r+t");
    if (iStream==NULL) {
        wsprintf(line,"Error while opening file %s ",(LPSTR)name);
        MessageBox(hDemoWin,line,NULL,MB_OK);
        return FALSE;
    }

    READ_LINE:
    result=fgets(line,300,iStream);
    if (ferror(iStream)) {
       wsprintf(line,"Error while reading file %s ",(LPSTR)name);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    if (feof(iStream))   goto END_FILE; // end of file 
    if (result==NULL) {
       wsprintf(line,"Error while reading file %s ",(LPSTR)name);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    
    CurLen=strlen(line);             // exclude new line character 
    if (CurLen>0 && line[CurLen-1]==0xA) CurLen--;
    line[CurLen]=0;

    LineIdx=0;                       // prepare to scan the line 

    // extract the field name 
    if (!ExtractField(line,&LineIdx,CurLen,CharReturn)) { // extract field name 
       wsprintf(line,"Invalid format, file: %s, field# %d\n",(LPSTR)name,1);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    StringTrim(CharReturn);              // trim space 
    strcpy(FieldDef[CurField].ShortName,CharReturn);
    strcpy(FieldDef[CurField].FullName,"CUSTOMER->");
    strcat(FieldDef[CurField].FullName,CharReturn);

    // extract the field width 
    if (!ExtractField(line,&LineIdx,CurLen,CharReturn)) { // extract field name 
       wsprintf(line,"Invalid format, file: %s, field# %d\n",(LPSTR)name,1);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    FieldDef[CurField].width=atoi(CharReturn);

    // extract the field type 
    if (!ExtractField(line,&LineIdx,CurLen,CharReturn)) { // extract field name 
       wsprintf(line,"Invalid format, file: %s, field# %d\n",(LPSTR)name,1);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }

    if      (CharReturn[0]=='T') FieldDef[CurField].type=TYPE_TEXT;
    else if (CharReturn[0]=='N') FieldDef[CurField].type=TYPE_NUM;
    else if (CharReturn[0]=='F') FieldDef[CurField].type=TYPE_DBL;
    else if (CharReturn[0]=='D') FieldDef[CurField].type=TYPE_DATE;
    else if (CharReturn[0]=='R') FieldDef[CurField].type=TYPE_RADIO;
    else if (CharReturn[0]=='C') FieldDef[CurField].type=TYPE_CHECKBOX;
    else if (CharReturn[0]=='P') FieldDef[CurField].type=TYPE_PICT;
    else {
       wsprintf(line,"Invalid field type, file: %s, field# %d\n",(LPSTR)name,1);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }

    // get the group number for the radio button fields
    if (CharReturn[0]=='R') FieldDef[CurField].RadioGroup=atoi(&(CharReturn[1]));

    // extract the default decimal places 
    if (!ExtractField(line,&LineIdx,CurLen,CharReturn)) { // extract field name 
       wsprintf(line,"Invalid format, file: %s, field# %d\n",(LPSTR)name,1);
       MessageBox(hDemoWin,line,NULL,MB_OK);
       return FALSE;
    }
    FieldDef[CurField].DecPlaces=atoi(CharReturn);

    CurField++;
    
    goto READ_LINE;

    END_FILE:
    fclose(iStream);
    TotalFields=CurField;

    return TRUE;
}

/******************************************************************************
    ExtractField:
    This routine scans a text line to extract the next field.
    The fields are assumed to be of character type.
******************************************************************************/
ExtractField(char *line,int *idx,int LineLen,char *CharReturn)
{
    char quote='"',comma=',',string[300],*ptr,SaveChar;
    int  i,j;

    i=(*idx);
    while (i<LineLen && line[i]==' ') i++;  // go past the spaces 

    if (i>=LineLen) return FALSE;           // past the end of the line 

    if (line[i]==quote) {
       i++;                                 // skip over the first quote 
       if (NULL==(ptr=memchr(&line[i],quote,LineLen-i))) return FALSE;
       
       j=1;
       while (ptr[j]!=',' && ptr[j]!=0) j++;// locate the next comma 
       
       SaveChar=ptr[0];                     // extract the string 
       ptr[0]=0;        
       strcpy(string,&line[i]);
       ptr[0]=SaveChar;
       (*idx)=i+strlen(string)+1+j;         // index to the next field        
    }
    else {
       if (NULL==(ptr=memchr(&line[i],comma,LineLen-i))) ptr=&line[LineLen];  // field spans the end of the line 
       
       SaveChar=ptr[0];                     // extract the string 
       ptr[0]=0;        
       strcpy(string,&line[i]);
       ptr[0]=SaveChar;
       (*idx)=i+strlen(string)+1;           // index to the next field        
    }

    strcpy(CharReturn,string);
    return TRUE;
}


/******************************************************************************
    Call back routines.
    The following routines are called by Report Ease to select an application
    field or to verify an application field.
******************************************************************************/

/******************************************************************************
    UserFieldSelection:
    This routine is called by the FORM_FLD module to allow user to select 
    a data field.  This routine can be programmed by your application in
    any way as long as it returns the data about the selected field using
    the argument pointer.  If the user chose not to select a field after all,
    the function should return with a FALSE value. Otherwise it should return
    with a TRUE value.

    The required data about the selected field is filled into the 'field' 
    structure. Your program may also optionally fill other remaining 
    'field' structure variables.

    The first parameter contains the handle of the Form Editor window.
******************************************************************************/
int FAR PASCAL _export UserFieldSelection(HWND hWnd,struct StrField huge *field)
{
    int  CurField;
    LPVOID lpProc;

    // SELECT FIELD
    lpProc = MakeProcInstance(DemoFileField, hInst);
    CurField=DialogBoxParam(hInst,"DemoFileField",hWnd,lpProc,0L);
    FreeProcInstance(lpProc);              // set back the old window 
    
    if (CurField<0) return FALSE;

    // fill the required 'field' variables 
    lstrcpy(field->name,FieldDef[CurField].FullName); // field name 
    field->type=FieldDef[CurField].type;             // alpha/num etc 
    field->width=FieldDef[CurField].width;           // display width 
    field->DecPlaces=FieldDef[CurField].DecPlaces;   // decimal places for display 
    field->RadioGroup=FieldDef[CurField].RadioGroup; // group id for the radio fields

    // Set the FLAG_CUSTOM_DATA flag for the 'rating' field.  The demo program
    // is assuming the responsibility for the data entry of this field.
    if (lstrcmpi(field->name,"customer->rating")==0) field->flags|=FLAG_CUSTOM_DATA;

    // specify new paragraph indicator field.  Used only for a word/wrapped
    // text type fields.
    field->ParaChar[0]='|';

    field->FieldId=CurField;  // optional information only fields
    
    return TRUE;
}

/******************************************************************************
    VerifyData:
    This callback function is called in the 'run' mode afer user enters
    some data into a data field. The user data is returned in the following
    member variable of the field structure:
    
    NumData:  Data for the field types: numeric, date, radiobutton, checkbox
    DblData:  Data for the floating numeric field
    CharData: Pointer to the text type data
    
    This function should return TRUE if the field is valid, otherwise it returns
    a FALSE value.

    This function is also called when a pushbutton field is clicked. This
    allows your application to take appropriate action on a pushbutton click.
    The result value is not used when a pushbutton is clicked.

    During this function your application can also populate other fields
    in the form.
******************************************************************************/
BOOL FAR PASCAL _export VerifyData(HWND hWnd,struct StrField huge *field)
{
    BYTE CustId[10],string[300];

    if (field->source==SRC_SYS) {    // handle the system pushbutton events
      // make sure that the customer id field exists in this form
      if (!FrsGetFieldData(hWnd,"CUSTOMER->CUST_ID",0,NULL,NULL,CustId)) {
         FrsShowMessage(hWnd,"Customer Id Field Missing!");
         return FALSE;
      }

      // Locate and show the next customer
      if (field->SysIdx==SYS_FIND_NEXT) {
         if (!ReadNextRecord(CustId)) {
            FrsShowMessage(hWnd,"No More Customers Found!");
            return FALSE;
         }
         ShowRecord(hWnd,FALSE);          // show the current record
         return TRUE;
      }
      // locate and show the previous customer
      else if (field->SysIdx==SYS_FIND_PREV) {
         if (!ReadPrevRecord(CustId)) {
            FrsShowMessage(hWnd,"No More Customers Found!");
            return FALSE;
         }
         ShowRecord(hWnd,FALSE);          // show the current record
         return TRUE;
      }
      // View the data for the current customer
      else if (field->SysIdx==SYS_VIEW_DATA) {
         if (!ReadRecord(CustId)) {
            FrsShowMessage(hWnd,"Invalid Customer Id!");
            return FALSE;
         }
         ShowRecord(hWnd,FALSE);          // show the current record
         return TRUE;
      }
      // Add the customer id in the data file
      else if (field->SysIdx==SYS_ADD_KEY) {
         if (lstrlen(CustId)==0)  {
            FrsShowMessage(hWnd,"Blank Customer Id Not Allowed");
            return FALSE;
         }
         // check if the customer id already in the database
         if (ReadRecord(CustId)) {
            wsprintf(string,"Customer Id: %s already Exists!",(LPSTR)CustId);
            FrsShowMessage(hWnd,string);
            return FALSE;
         }
         if (WriteRecord(hWnd,CustId,'A')) {
            FrsShowMessage(hWnd,"Customer Record Added");
            return TRUE;
         }
         else {
            FrsShowMessage(hWnd,"Error adding Customer Id!");
            return FALSE;
         }
      }
      // Verify the customer id in the data file
      else if (field->SysIdx==SYS_VERIFY_KEY) {
         if (ReadRecord(CustId)) {
            FrsShowMessage(hWnd,"Customer Record Found");
            return TRUE;
         }
         else {
            FrsShowMessage(hWnd,"Customer Record Not Found!");
            return FALSE;
         }
      }
      // Update the data for the current customer
      else if (field->SysIdx==SYS_UPDATE_DATA) {
         if (!ReadRecord(CustId)) {
            FrsShowMessage(hWnd,"This Customer does Not exist in the Data File!");
            return FALSE;
         }
         if (WriteRecord(hWnd,CustId,'U')) {
            FrsShowMessage(hWnd,"Record Updated");
            return TRUE;
         }
         else {
            FrsShowMessage(hWnd,"Update Not Successful!");
            return FALSE;
         }
      }
      // Delete the current record
      else if (field->SysIdx==SYS_DELETE_DATA) {
         if (!ReadRecord(CustId)) {
            FrsShowMessage(hWnd,"This Customer does Not exist in the Data File!");
            return FALSE;
         }
         if (WriteRecord(hWnd,CustId,'D')) {
            FrsShowMessage(hWnd,"Record Deleted");
            return TRUE;
         }
         else {
            FrsShowMessage(hWnd,"Delete Not Successful!");
            return FALSE;
         }
      }
    
    }
                            
    return TRUE;
}

/******************************************************************************
    EnterData:
    This callback function is called in the 'run' mode when the user
    clicks on a field that has the FLAG_CUSTOM_DATA flag set.  This flag
    can be set by your application in the UserFieldSelection function.
    This function allows the user to enter the data for the field.
    The user entered data is returned to the form executer in the following
    member variable of the field structure:
    
    NumData:  Data for the field types: numeric, date, radiobutton, checkbox, picture id
    DblData:  Data for the floating numeric field
    CharData: Pointer to the text type data
    
    This function should return TRUE if the users enters the data, otherwise
    it should return a FALSE value.

    In this example, the customer->rating field is set as custom data.  Here
    we will show a list box with the selection for the rating.
******************************************************************************/
BOOL FAR PASCAL _export EnterData(HWND hWnd,struct StrField huge *field, int x, int y, int width, int height)
{
    LPVOID lpProc;
    BOOL result;


    if (lstrcmpi(field->name,"customer->rating")==0) {
        // set the global variables to pass data to the dialog box
        lstrcpy(DlgText,field->CharData);    // rating
        DlgInt1=x;                           // x location of the 
        DlgInt2=y;                           // y location
        
        // Select the rating
        lpProc = MakeProcInstance(DemoRating, hInst);
        result=DialogBoxParam(hInst,"DemoRating",hWnd,lpProc,0L);
        FreeProcInstance(lpProc);              // set back the old window 
    
        if (result) lstrcpy(field->CharData,DlgText);  // pass the field value
        return result;
    }


    return FALSE;
}

/******************************************************************************
    DrawPicture:
    This routine is called by the form executor to draw a specified
    picture type field.  The first argument specifies the device context
    of the reporting device. You application should draw the picture
    on this device context within the specified rectangle (last argument).

    The device context is in ANISOTROPIC mode.  The resolution in the X and
    Y direction is given by the UNITS_PER_INCH constant.

    The third argument is the picture id. The fourth argument is the Field Id
    for the this picture field.

    The function returns TRUE if successful.

    In this demo program, this routine draws a logo for a given picture id.
    This program uses only one bitmap, and draws different part of the same
    bitmap for different picture id.

******************************************************************************/
int FAR PASCAL _export DrawPicture(HWND hWnd, HDC hDC, int PictId, int FieldId, RECT far * rect)
{
    BITMAP bm;
    int SourceWidth,SourceX;

    if (PictId==0) return FALSE;

    // get the bitmap information
    GetObject(hLogoBM,sizeof(BITMAP),&bm);

    // Select part of the bitmap to display for this picture id
    SourceWidth=bm.bmWidth/10;          // divide picture into 10 equal parts
    if (PictId<1)  PictId=1;
    if (PictId>10) PictId=10;
    SourceX=SourceWidth*(PictId-1);

    // copy the bitmap
    StretchDIBits(hDC,rect->left,rect->top,rect->right-rect->left,rect->bottom-rect->top,
                   SourceX,0,SourceWidth,bm.bmHeight,pImage,(LPBITMAPINFO)pInfo,DIB_RGB_COLORS,SRCCOPY);

    return TRUE;
}

/******************************************************************************
    Support Routines.
******************************************************************************/

/*****************************************************************************
    StringTrim:
    Remove spaces from the left and right of a NULL terminated string.
******************************************************************************/
void StringTrim(LPSTR string)  
{
    RightTrim(string);LeftTrim(string);
}

/******************************************************************************
    RightTrim:
    Remove spaces on the right of a string.
******************************************************************************/
void RightTrim(LPSTR string)  
{
    int i,TempLen;

    TempLen=lstrlen(string);
    for (i=TempLen-1;i>=0 && string[i] == ' ';i--);
    string[i+1] = '\0';
}

/******************************************************************************
    LeftTrim:
    Trim initial spaces from the string.
*******************************************************************************/
void LeftTrim(LPSTR string)    
{
    int i,TempLen,BeginPoint;char TempStr[300];

    TempLen=lstrlen(string);
    for (i=0;i < TempLen && string[i] == ' ';i++);
    BeginPoint=i;
    for (i=BeginPoint;i<TempLen;i++) TempStr[i-BeginPoint]=string[i];
    TempStr[TempLen-BeginPoint] = '\0';
    lstrcpy(string,TempStr);
}

/******************************************************************************
     GetNumVal:
     Extract a numeric value from the dialog item.  Display error if a non-
     numeric value is encountered and return a false result.
*******************************************************************************/
BOOL GetNumVal(HWND hWin,int ControlId,int *pInt)
{
    BOOL ErrorFlag;

    *pInt=GetDlgItemInt(hWin,ControlId,&ErrorFlag,(BOOL) 0);
    if (!ErrorFlag) {
       MessageBox(hWin,"Must Enter a Valid Number",NULL,MB_OK);
       SetFocus(GetDlgItem(hWin,ControlId));
       return (FALSE);
    }
    return TRUE;
}

/******************************************************************************
     Bitmap2DIB:
     Extract device indepedent bits from the bitmap.
*******************************************************************************/
Bitmap2DIB()
{
    DWORD  InfoSize,ImageSize;
    BITMAP bm;
    HDC hDC;
    long ScanLineSize;

    GetObject(hLogoBM,sizeof(BITMAP),(LPSTR)&bm);     // get the dimension of bit map 
   
    //********** create the device independent bitmap ***********************
    InfoSize=sizeof(BITMAPINFOHEADER)+256*sizeof(RGBQUAD);  // create a 256 color bitmap structure 

    if (NULL==(hInfo=GlobalAlloc(GMEM_MOVEABLE,InfoSize))
      ||NULL==(pInfo=(LPBITMAPINFOHEADER)GlobalLock(hInfo)) ) {
        MessageBox(hDemoWin,"Ran out of memory for Bitmap Info!",NULL,MB_OK);
        return -1;
    }

    //***** fill in the info structure ******
    pInfo->biSize=sizeof(BITMAPINFOHEADER);
    pInfo->biWidth=bm.bmWidth;
    pInfo->biHeight=bm.bmHeight;
    pInfo->biPlanes=1;                       // DIB have plane = 1 
    pInfo->biBitCount=8;                     // 8 bits per pixel or color 
    pInfo->biCompression=BI_RGB;             // no compression - needed for compatibility for all devices
    ScanLineSize=((pInfo->biWidth+sizeof(LONG)-1)/sizeof(LONG))*sizeof(LONG); // size of each scan line
    pInfo->biSizeImage=ScanLineSize*pInfo->biHeight; // buffer size requirement
    pInfo->biXPelsPerMeter=0;
    pInfo->biYPelsPerMeter=0;
    pInfo->biClrUsed=0;                      // depends on biBitCount 
    pInfo->biClrImportant=0;                 // all colors important 

    hDC=GetDC(hDemoWin);
    ImageSize=pInfo->biSizeImage;
    
    if (NULL==(hImage=GlobalAlloc(GMEM_MOVEABLE,ImageSize))
      ||NULL==(pImage=GlobalLock(hImage)) ) {
        MessageBox(hDemoWin,"Ran out of memory for Bitmap Image!",NULL,MB_OK);
        return -1;
    }

    if (!GetDIBits(hDC,hLogoBM,0,bm.bmHeight,pImage,(LPBITMAPINFO)pInfo,DIB_RGB_COLORS)) {
        MessageBox(hDemoWin,"Error Creating Device Independent Bitmap Image!",NULL,MB_OK);
        return -1;
    }
    
    // release the common device context
    ReleaseDC(hDemoWin,hDC);

    return TRUE;
}

/*****************************************************************************
    ModifyExt:
    This function modifies the extension of a given file (argument #1) to
    as specified by the second argument.
******************************************************************************/
ModifyExt(char *InputFile,char *NewExt)
{
    #if defined(__TURBOC__)          /* compiler specific code */
        char drive[MAXDRIVE],dir[MAXDIR],FileName[MAXFILE],FileExt[MAXEXT];

        fnsplit(InputFile,drive,dir,FileName,FileExt);
        fnmerge(InputFile,drive,dir,FileName,NewExt);
    #else
        char drive[_MAX_DRIVE],dir[_MAX_DIR],FileName[_MAX_FNAME],FileExt[_MAX_EXT];
        _splitpath(InputFile,drive,dir,FileName,FileExt);
        _makepath(InputFile,drive,dir,FileName,NewExt);
    #endif
    
    return TRUE;
}



/******************************************************************************
    OurPrintf:
    This routine formats and display a given set of arguments.  The format
    is given by the first argument.  The argument 2 to n contain the
    arguments for the specified format.  The function uses MessageBox to
    display the formatted string.
******************************************************************************/
OurPrintf(LPSTR fmt,...)
{
    LPSTR ArgStart;
    char string[256];

    ArgStart=(LPSTR) &fmt;  // pointer to first argument 
    ArgStart=&ArgStart[4];  // go past the first argument 
    wvsprintf(string,fmt,ArgStart);
    if (FindWindow("DBWin",NULL)) { // debug window open
        lstrcat(string,"\n");
        OutputDebugString(string);  // send to the debug terminal
    }
    else MessageBox(NULL,string,NULL,MB_OK);
    return TRUE;
}

/******************************************************************************
    DoubleToStr:
    This routine converts a double value to a string.  The second argument
    provides the digits after the decimal point.  
    The function returns the pointer to the third argument after storing
    the result in the third argument.
*******************************************************************************/
LPSTR DoubleToStr(double nbr,int dec,LPSTR out)
{
    long i,len,BeforeDec,AfterDec,multiply;
    char string[20];
    double dAfterDec;
    
    out[0]=0;

    //************* apply sign *******
    if (nbr<0) {
       lstrcat(out,"-");
       nbr=-nbr;
    }

    BeforeDec = (long) nbr;           // number before the decimal point 

    //****** append the digits after the decimal point ****
    if (dec>0) {
       nbr=nbr-(double)BeforeDec;
       multiply=1;                   
       for (i=0;i<dec;i++) multiply=multiply*10;
       dAfterDec=nbr*(double)multiply;
       AfterDec=(long)dAfterDec;
       nbr=dAfterDec-(double)AfterDec;
       if (nbr>=.5) AfterDec++;
       if (AfterDec>=multiply) {    // overflow 
          BeforeDec++;
          AfterDec=0;
       }
       
       //************* make the out *****************
       if (BeforeDec>0) {
          ltoa(BeforeDec,string,10);
          lstrcat(out,string);
       }
       lstrcat(out,".");
       
       ltoa(AfterDec,string,10);
       len=strlen(string);
       for (i=len;i<dec;i++) lstrcat(out,"0");  // first pad with necessary space
       lstrcat(out,string);                     // then append the digits
    }
    else if (BeforeDec>0) {
       ltoa(BeforeDec,string,10);
       lstrcat(out,string);
    }
    
    return out;
}

