Attribute VB_Name = "FRS"

'*****************************************************************************
'                      Global Constants
'*****************************************************************************

Global Const UNITS_PER_INCH = 250

Global Const FORM_SIGN = &HDFBA&
Global Const NAME_WIDTH = 50

Global Const SRC_APPL = 0
Global Const SRC_SYS = 1
Global Const SRC_CONST = 2
Global Const SRC_NONE = 3

Global Const TYPE_NUM = 1
Global Const TYPE_DBL = 2
Global Const TYPE_TEXT = 3
Global Const TYPE_DATE = 4
Global Const TYPE_PICT = 5
Global Const TYPE_RADIO = 6
Global Const TYPE_CHECKBOX = 7
Global Const TYPE_PUSHBUTTON = 8

Global Const DT_MMDDYY = 0
Global Const DT_DDMMYY = 1
Global Const DT_MMDDYYYY = 2
Global Const DT_MMMDDYYYY = 3

Global Const FLAG_TRIM = &H1
Global Const FLAG_SUP_ZERO = &H2
Global Const FLAG_PAD_ZERO = &H4
Global Const FLAG_CAPS = &H8
Global Const FLAG_FIRST_CAP = &H10
Global Const FLAG_SMALL = &H20
Global Const FLAG_COMMA = &H40
Global Const FLAG_WRAP = &H80
Global Const FLAG_WORD_WRAP = &H100
Global Const FLAG_CUSTOM_DATA = &H200
Global Const FLAG_FLEX_SIZE = &H400
Global Const FLAG_SHARED = &H1000
Global Const FLAG_PROTECT = &H2000
Global Const FLAG_3D_OUTLINE = &H4000

Global Const FFLAG_RETURN_MSG_ID   = &H1
Global Const FFLAG_SEND_SYS_FIELD_MSG = &H2

Global Const SYS_DATE       = 0
Global Const SYS_TIME       = 1
Global Const SYS_ADD_KEY    = 2
Global Const SYS_VERIFY_KEY = 3
Global Const SYS_VIEW_DATA  = 4
Global Const SYS_UPDATE_DATA  = 5
Global Const SYS_DELETE_DATA  = 6
Global Const SYS_FIND_NEXT    = 7
Global Const SYS_FIND_PREV    = 8
Global Const SYS_RESET        = 9
Global Const MAX_SYS_FIELDS   = 10

Global Const PAINT_ITEMS      = 2
Global Const PAINT_WIN        = 3
Global Const PAINT_PARTIAL_WIN = 5

Global Const REP_CLOSE = &H400

Global Const ERR_NO_FILE = 1
Global Const ERR_NO_DEVICE = 2
Global Const ERR_NO_MEM = 3
Global Const ERR_BAD_EXP = 4
Global Const ERR_SUSPEND = 5
Global Const ERR_ZERO_DIVIDE = 6
Global Const ERR_TYPE_MISMATCH = 7
Global Const ERR_INCOMPLETE = 8
Global Const ERR_DUPLICATE = 9
Global Const ERR_PRINTER = 10
Global Const ERR_IO = 11
Global Const ERR_DISPLAY = 12
Global Const ERR_WRAP = 13
Global Const ERR_FONT = 14
Global Const ERR_OTHER = 15

Global Const ERR_BAD_WND = 16
Global Const ERR_NO_FRS_ID = 17
Global Const ERR_BAD_FRS_ID = 18
Global Const ERR_NO_OUTFILE = 19
Global Const ERR_NO_TER = 20
Global Const ERR_IN_TER = 21


Global Const WS_OVERLAPPEDWINDOW = &HCF0000
Global Const WS_CHILD = &H40000000
Global Const MF_ENABLED = 0
Global Const MF_GRAYED = 1
Global Const MF_CHECKED = 8
Global Const MF_UNCHECKED = 0

'******************************************************************************
'                       Form Header Structure
'******************************************************************************
Type TypeFormHdr
       FormSign As Long       ' Form File Signature
       
       name As String * 52       ' report name (NAME_WIDTH+2)
                                       ' This field is NULL terminated

       DataSetName As String * 20 ' Your application can specify a data set
                                 ' name which can be used to associate the
                                 ' application data to the form.  This
                                 ' is an optional information.
                                 ' This field is NULL terminated
       
       TotalItems As Long     ' screen items in the form template
       FieldCount As Long     ' total number of fields
       FontCount As Long      ' number of entries in the font table
       SectionCount As Long  ' number of parts in the form

       LeftMargin As Single      ' margins in inches
       RightMargin As Single
       TopMargin As Single
       BottomMargin As Single

       PrinterName As String * 52 ' name of the printer 2*(NAME_WIDTH+2)
       PrinterDriver As String * 52 ' printer driver name 2*(NAME_WIDTH+2)

       flags As Long          ' RFLAG_ series flags

       DateFormat As Long     ' 0=MM/DD/YY, 1=DD/MM/YY
       
       RulerType As Long      ' RULER_OFF,RULER_INCH,RULER_CM
       
       SecBannerHeight As Long ' height (logical units) of the section banner
       FileFormatId As Long    ' records the file format changes
       HdrReserved1 As String * 142 ' reserved bytes
End Type

'******************************************************************************
'                        Field Structure
'******************************************************************************
Type TypeField                    ' define each field
       source As Long        ' source of field data: application,calculation,system, constant etc

       name As String * 52      ' field name (NAME_WIDTH+2)
                                ' Append chr (0) to NULL terminate this field

       FieldId As Long       ' another optional id that may be defined by the calling application
       type As Long          ' numeric/alphanumeric/logical/date
       width As Long         ' field data width as stored in your database
       DecPlaces As Long     ' decimal places for numeric fields
       AllowChanges As Long  ' Allow modification of the following fields

       InUse As Long         ' TRUE=in use, FALSE=not in use
       flags As Long         ' flags
       SysIdx As Long        ' index into the system field table for system fields

       DateFormat As Long    ' date format
       
       DateDelim As String * 2  ' date delimiter character, i.e. / or -
       
       ParaChar As String * 2   ' specify the new paragraph indicator character
                                ' in the first byte.  Needed only for a word/wrapped
                                ' text field.
                                ' You must append chr(0) to NULL terminate this field

       CurrencySymbol As String * 4 ' currency symbol

       NegSignPrefix As String * 4 ' negative sign prefix
       NegSignSuffix As String * 4 ' negative sign suffix
       PosSignPrefix As String * 4 ' positive sign prefix
       PosSignSuffix As String * 4 ' positive sign suffix

       reserved1 As Long        ' character data, pointer to an object of size = field.width+1
       NumData As Long          ' numeric data (long)
       DblData As Double        ' numeric data for field = TYPE_DBL

       RadioGroup As Long    ' group number for the radio fields

       section As Long       ' section that holds the field (report time only)
       misc As Long          ' used for miscellaneous purposes

       UseCount As Long      ' use count for the shared fields

       FldReserved1 As String * 10 ' reserved to pad to 128
End Type

'******************************************************************************
'                        Picture information structure
'******************************************************************************
Type TypePict
      hDC     As Long        ' Device Context of the device
      PictId  As Long        ' picture id value
      FieldId As Long        ' FieldId of the picture
      x       As Long        ' picture location
      y       As Long        ' picture location
      width   As Long        ' picture width
      height  As Long        ' picture height
End Type

'******************************************************************************
'                        Command ids
'******************************************************************************
Global Const ID_PGUP         =600
Global Const ID_PGDN         =601
Global Const ID_UP           =602
Global Const ID_DOWN         =603
Global Const ID_LEFT         =604
Global Const ID_RIGHT        =605
Global Const ID_INSERT_LABEL =606
Global Const ID_MOVE_ITEM    =608
Global Const ID_SIZE_ITEM    =609
Global Const ID_CENTER_ITEM  =610
Global Const ID_SELECT_MULTIPLE =611
Global Const ID_EDIT_LABEL   =612
Global Const ID_PICT_FROM_CB =620
Global Const ID_PICT_FROM_FILE =621
Global Const ID_HELP         =622
Global Const ID_SAVE         =624
Global Const ID_SAVEAS       =625
Global Const ID_QUIT         =626
Global Const ID_FORM_PARAM    =628
Global Const ID_PRINT_OPTIONS =630

Global Const ID_FONTS        =638

Global Const ID_EDIT_FLD     =639

Global Const ID_INSERT_DATA  =641
Global Const ID_INSERT_SYS   =643
Global Const ID_SEC_NEW      =645
Global Const ID_SEC_EDIT     =646

Global Const ID_INSERT_LINE  =660
Global Const ID_ITEM_OUTLINE =661
Global Const ID_ITEM_BACKGROUND =662
Global Const ID_POS_ITEM     =663
Global Const ID_DEL_ITEM     =664
Global Const ID_UNDO         =665
Global Const ID_EDIT_LINE    =666

Global Const ID_ALIGN_HORZ_TOP    =675
Global Const ID_ALIGN_HORZ_BOT    =676
Global Const ID_ALIGN_HORZ_CENTER =677
Global Const ID_ALIGN_VERT_LEFT   =678
Global Const ID_ALIGN_VERT_RIGHT  =679
Global Const ID_ALIGN_VERT_CENTER =680

Global Const ID_EVEN_SIZE_HORZ    =681
Global Const ID_EVEN_SIZE_VERT    =682

Global Const ID_EVEN_SPACE_HORZ   =683
Global Const ID_EVEN_SPACE_VERT   =684

Global Const ID_EXPAND_HORZ       =690
Global Const ID_EXPAND_VERT       =691
Global Const ID_COMPRESS_HORZ     =692
Global Const ID_COMPRESS_VERT     =693

Global Const ID_JUMP         =875
Global Const ID_PRINT        =876
Global Const ID_PREVIEW      =877
Global Const ID_EXIT         =879
                             
Global Const ID_DESC_WND     =900
Global Const ID_CUT          =901
Global Const ID_COPY         =902
Global Const ID_PASTE        =903
Global Const ID_SNAP_TO_GRID =904

Global Const ID_INSERT_PART  =905
Global Const ID_APPEND_PART  =906
Global Const ID_DEL_PART     =907

Rem *****************************************************************************
Rem                    Function Prototypes for the DLL functions
Rem ******************************************************************************
Declare Function FrsShowMessage Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal msg As String) As Long
Declare Function FrsGetFieldData Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal FieldName As String, ByVal FieldId As Long, NumData As Long, DbData As double, ByVal CharData As String) As Long
Declare Function FrsSetFieldData Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal FieldName As String, ByVal FieldId As Long, ByVal NumData As Long, ByVal DbData As double, ByVal CharData As String, ByVal repaint As Long) As Long
Declare Function FrsRepaint Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal PaintFlag As Long) As Long
Declare Function FrsGetModified Lib "FRS32.DLL" (ByVal hWnd As Long) As Long
Declare Function FrsSetModified Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal SetReset As Long) As Long
Declare Function FrsMenuEnable Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal CommandId As Long) As Long
Declare Function FrsMenuSelect Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal CommandId As Long) As Long
Declare Function FrsSetFlags Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal SetReset As Long, ByVal flag As Long) As Long
Declare Function FrsRead Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal FileName As String) As Long
Declare Function FrsSave Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal FileName As String) As Long
Declare Function FrsCommand Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal CommandId As Long) As Long
Declare Function FrsQueryExit Lib "FRS32.DLL" (ByVal hWnd As Long) As Long
Declare Function FrsGetSelectedField Lib "FRS32.DLL" (ByVal hWnd As Long, field As TypeField) As Long
Declare Function FrsSetSelectedField Lib "FRS32.DLL" (ByVal hWnd As Long, field As TypeField) As Long
Declare Function FrsSetField Lib "FRS32.DLL" (ByVal hWnd As Long, ByVal Fieldname as String, ByVal FieldId as long, field As TypeField, ByVal SetGet as long) As Long
Declare Function FrsSetLicenseKey Lib "FRS32.DLL" (ByVal LicKey As String) As Long


Rem *****************************************************************************
Rem                    Function Prototypes for the OCX functions
Rem ******************************************************************************
Declare Function FvbGetFieldInfo Lib "FOC.OCX" (ByVal hWnd As Long, field As TypeField) As Long
Declare Function FvbSetFieldInfo Lib "FOC.OCX" (ByVal hWnd As Long, field As TypeField, ByVal valid As Long) As Long
Declare Function FvbGetPictureInfo Lib "FOC.OCX" (ByVal hWnd As Long, PictInfo As TypePict) As Long
Declare Function FvbDrawBitmap Lib "FOC.OCX" (ByVal hWnd As Long, ByVal hWnd As Long, ByVal hBM As Long, ByVal PictX As Long, ByVal PictY As Long, ByVal PictWidth As Long, ByVal PictHeight As Long) As Long
Declare Function FvbEnableFormWindow Lib "FOC.OCX" (ByVal hWnd As Long, ByVal enable As Long) As Long


Rem **************************************************
Rem When a string is passed for OUTPUT to the DLL
Rem ,the output string created by the DLL has an extra
Rem NULL character.  For such strings, you must call
Rem this routine to strip the null character from the
Rem given string
Rem **************************************************
Function DiscardNull(InText As String) As String
   Dim i As Long
   Dim StrLen As Long
   Dim CurChar As String
   
   StrLen = Len(InText)

   For i = 1 To StrLen
      CurChar = Left(InText, i)
      CurChar = Right(CurChar, 1)

      If Asc(CurChar) = 0 Then
         DiscardNull = Left(InText, i - 1)
         Exit Function
      End If
   Next i

   DiscardNull = InText
                       
End Function

